// 引入需要的模块
import { createRouter,  createWebHistory } from 'vue-router'
import {getUserInfo, login} from "../common/api/user/index.js";
import {getToken, isAdmin} from "../common/utils/auth.js";
import {ElMessage} from "element-plus";

// 下面使用了es6的对象增强写法，命名必须是routes
const routes = [
    {
        path: '/',
        redirect: 'index'
    },
    {
        path: '/login',
        name:'login',
        // 已经配置了路径别名，@/view 就可以写成 view
        // 配置了extensions，login.vue可以写成login
        component: () => import('@/views/login'),
        beforeEnter:(to, from, next) =>{
             // 我们也可以自定义鉴权等逻辑操作
            next();
         }
    },
    {
        path: '/index',
        name: 'index',
        component: () => import('@/views/home'),
        meta:{
            // 自定义一个字段,标识需要进行鉴权
            logined:false
        },
        children:[
            {
                path: '/index/chart',
                name:'chart',
                component: () => import('@/views/myChart'),
                beforeEnter:(to, from, next) =>{
                    next();
                }
            },
            {
                path: '/index/userManage',
                name:'userManage',
                component: () => import('@/views/userManage'),
                beforeEnter:(to, from, next) =>{
                    if (isAdmin()){
                        next();
                    }
                }
            },
            {
                path: '/index/chartManage',
                name:'chartManage',
                component: () => import('@/views/chartManage'),
                beforeEnter:(to, from, next) =>{
                    if (isAdmin()){
                        next();
                    }
                }
            }
        ]
    }
]

// 创建路由
const router = createRouter({
    history: createWebHistory(),
    routes
})

// 全局前置路由守卫
router.beforeEach( (to,from,next)=>{
    // to是目标路由对象,from当前路由对象
        next();
        // getUserInfo().then(res => {
        //     console.log(res)
        //     // if (res.code == 200 && to.fullPath == '/login'){
        //     //     router.replace({
        //     //         path: '/',
        //     //     });
        //     // }else if(res.code == 200){
        //     //     next();
        //     // }else {
        //     //     ElMessage({
        //     //         message: '请先登录',
        //     //         type:'warning'
        //     //     })
        //     //     router.replace({
        //     //         path: '/login',
        //     //     })
        //     // }
        // });
    // next()
});

//全局后置路由守卫
router.afterEach((to,from)=>{
    // to是目标路由对象,from当前路由对象
    // console.log(to,from);
    // 后置路由守卫没有next函数,因为已经切换路由,不需要进行放行
    // 我们可以在这里进行一些后续的善后操作,比如设置切换路由后网页的标题
});

// 导出路由
export default router
