package space.anyi.BI.controller;

import cn.hutool.core.util.IdUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import space.anyi.BI.entity.Chart;
import space.anyi.BI.entity.ResponseResult;
import space.anyi.BI.entity.dto.ChartDTO;
import space.anyi.BI.entity.vo.ChartVO;
import space.anyi.BI.entity.vo.PageVO;
import space.anyi.BI.handler.redisson.RRateLimiterHandler;
import space.anyi.BI.service.ChartService;
import space.anyi.BI.util.SecurityUtils;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.List;

/**
 * @ProjectName: BI
 * @FileName: ChartController
 * @Author: 杨逸
 * @Data:2024/11/28 19:53
 * @Description:
 */
@Controller()
@RequestMapping("/chart")
public class ChartController {
    private final static Logger log = LoggerFactory.getLogger(ChartController.class);
    @Resource
    private ChartService chartService;
    @Resource
    private RRateLimiterHandler rRateLimiterHandler;
    @PreAuthorize("@ps.hasRole('用户')")
    @GetMapping("/getChartById/{id}")
    @ResponseBody
    public ResponseResult getChartById(@PathVariable("id") Long id){
        Chart chart = chartService.getById(id);
        return ResponseResult.okResult(chart);
    }
    @PreAuthorize("@ps.hasRole('管理员')")
    @PostMapping("/add")
    @ResponseBody
    public ResponseResult addChart(@RequestBody Chart chart){
        long id = IdUtil.getSnowflake(1, 1).nextId();
        chart.setId(id);
        Long userId = SecurityUtils.getUserId();
        chart.setUserId(userId);
        chartService.save(chart);
        return ResponseResult.okResult();
    }
    @PreAuthorize("@ps.hasRole('用户')")
    @DeleteMapping("/{ids}")
    @ResponseBody
    public ResponseResult deleteChartById(@PathVariable List<Long> ids){
        chartService.removeByIds(ids);
        return ResponseResult.okResult();
    }
    @PreAuthorize("@ps.hasRole('管理员')")
    @PutMapping("/update")
    @ResponseBody
    public ResponseResult updateChart(@RequestBody Chart chart){
        chartService.updateById(chart);
        return ResponseResult.okResult();
    }
    @PreAuthorize("@ps.hasRole('用户')")
    @GetMapping("/list")
    @ResponseBody
    public ResponseResult page(Integer pageNum,Integer pageSize,String name,Long UserId){
        PageVO pageVO  = chartService.getChartPage(pageNum,pageSize,name,UserId);
        return ResponseResult.okResult(pageVO);
    }

    @PreAuthorize("@ps.hasRole('用户')")
    @PostMapping("/generateChartByAI")
    @ResponseBody
    public ResponseResult generateChartByAI(ChartDTO chartDTO, MultipartFile file) throws IOException {
        log.info("分析目标:{}图标名称:{}",chartDTO.getAnalysisTarget(),chartDTO.getName());
        //限流判断
        if (!rRateLimiterHandler.accessAble("generateChartByAI_"+SecurityUtils.getUserId())){
            return ResponseResult.errorResult(ResponseResult.AppHttpCodeEnum.RATE_LIMIT_ERROR);
        }
        if (file == null || file.isEmpty()) {
            return ResponseResult.errorResult(ResponseResult.AppHttpCodeEnum.FILE_NOT_NULL);
        }
        //文件类判断
        if (!file.getContentType().equals("application/vnd.ms-excel") && !file.getContentType().equals("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet")) {
            return ResponseResult.errorResult(ResponseResult.AppHttpCodeEnum.FILE_TYPE_ERROR);
        }
        //文件大小判断
        if (file.getSize()>1024*1024*2L) {
            return ResponseResult.errorResult(ResponseResult.AppHttpCodeEnum.FILE_SIZE_ERROR);
        }
        ChartVO vo = chartService.generateChartByAI(chartDTO,file);

        return ResponseResult.okResult(vo);
    }

    @ExceptionHandler({IOException.class})
    private ResponseResult exceptionHandler(){
        return ResponseResult.errorResult(ResponseResult.AppHttpCodeEnum.FILE_ERROR);
    }
}
