package space.anyi.BI.entity.xinghuo;

import java.io.Serializable;

/**
 * 信息对象
 * @ProjectName: serve
 * @FileName: HttpRequestMessage
 * @Author: 杨逸
 * @Data:2024/12/4 16:41
 * @Description:
 */
public class HttpRequestMessage implements Serializable {

    /**
     * 系统命令
     * @see String
     */
    public static final String SYSTEM_ROLE = "system";
    /**
     * 用户
     * @see String
     */
    public static final String USER_ROLE = "user";
    /**
     * AI模型
     * @see String
     */
    public static final String ASSISTANT_ROLE = "assistant";
    /**
     * function call执行结果
     * @see String
     */
    public static final String TOOL_ROLE = "tool";
    private String role = USER_ROLE;
    private String content = "";

    public HttpRequestMessage(String role, String content) {
        this.role = role;
        this.content = content;
    }

    public HttpRequestMessage(String content) {
        this.content = content;
    }

    public HttpRequestMessage() {
    }

    /**
     * 获取预设的prompt信息
     * @return {@code HttpRequestMessage }
     * @description:
     * @author: 杨逸
     * @data:2024/12/04 20:52:46
     * @since 1.0.0
     */
    public static HttpRequestMessage getPromptMessage() {
        String prompt = "你是一位专业的数据分析师和前端工程师，擅长对数据进行分析并得出明确的结论。你精通使用Echarts进行数据可视化，并能结合可视化的数据图表进行合理的分析。\n" +
                "\n" +
                "**要求：**\n" +
                "1. 直接输出两个部分：Echarts图表的配置代码和数据分析结论，不包含其他提示内容。\n" +
                "2. 我将提供原始数据和分析目标，你需要根据这些信息生成Echarts图表的配置对象标准JSON代码。代码中不能使用不存在的变量和函数。\n" +
                "3. 输出格式为：\n" +
                "   - 首先输出Echarts的option配置对象的标准格式JSON代码。\n" +
                "   - 然后输出基于原始数据和图表分析得出的结论。\n" +
                "4. 请严格按照指定的输出格式提供所需内容。\n" +
                "\n" +
                "**输入格式：**\n" +
                "- 原始数据\n" +
                "- 分析目标\n" +
                "\n" +
                "**输出格式：**\n" +
                "- Echarts的option配置对象的JSON代码\n" +
                "- 数据分析结论";
        HttpRequestMessage message = new HttpRequestMessage(HttpRequestMessage.SYSTEM_ROLE,prompt);
        return message;
    }

    public String getRole() {
        return role;
    }

    public void setRole(String role) {
        this.role = role;
    }

    public String getContent() {
        return content;
    }

    public void setContent(String content) {
        this.content = content;
    }

    @Override
    public String toString() {
        return "HttpRequestMessage{" +
                "role='" + role + '\'' +
                ", content='" + content + '\'' +
                '}';
    }
}
