package space.anyi.BI.filter;

import com.alibaba.fastjson.JSON;
import io.jsonwebtoken.Claims;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.filter.OncePerRequestFilter;
import space.anyi.BI.BIApplication;
import space.anyi.BI.constant.SystemConstants;
import space.anyi.BI.entity.LoginUserDetails;
import space.anyi.BI.entity.ResponseResult;
import space.anyi.BI.util.JwtUtil;
import space.anyi.BI.util.RedisCache;
import space.anyi.BI.util.WebUtils;

import javax.annotation.Resource;
import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Objects;

/**
 * @ProjectName: BI
 * @FileName: JWTAuthenticationTokenFilter
 * @Author: 杨逸
 * @Data:2024/11/28 20:04
 * @Description:
 */
@Component
public class JWTAuthenticationTokenFilter extends OncePerRequestFilter {
    private final static Logger log = LoggerFactory.getLogger(BIApplication.class);
    @Resource
    private RedisCache redisCache;

    @Override
    protected void doFilterInternal(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, FilterChain filterChain) throws ServletException, IOException {
        //获取token
        String uri = httpServletRequest.getRequestURI();
        String token = httpServletRequest.getHeader("token");
        log.info("token={},\n请求的uri={}",token,uri);
        if (!StringUtils.hasText(token)) {
            filterChain.doFilter(httpServletRequest, httpServletResponse);
            return;
        }
        //解析token
        Claims jwt;
        try {
            jwt = JwtUtil.parseJWT(token);
        } catch (Exception e) {
            e.printStackTrace();
            //响应前端,需要登陆
            ResponseResult errorResult = ResponseResult.errorResult(ResponseResult.AppHttpCodeEnum.NEED_LOGIN);
            WebUtils.renderString(httpServletResponse, JSON.toJSONString(errorResult));
            return;
        }
        //从redis中获取用户信息
        LoginUserDetails loginUserDetails = redisCache.getCacheObject(SystemConstants.LOGIN_USER_REDIS_KEY +jwt.getSubject());

        //已退出登陆或登陆已过期
        if(Objects.isNull(loginUserDetails)){
            //响应前端,需要登陆
            ResponseResult errorResult = ResponseResult.errorResult(ResponseResult.AppHttpCodeEnum.NEED_LOGIN);
            WebUtils.renderString(httpServletResponse, JSON.toJSONString(errorResult));
            return;
        }

        //存入SecurityContextHolder
        Authentication authenticationToken = new UsernamePasswordAuthenticationToken(loginUserDetails,null,null);
        SecurityContextHolder.getContext().setAuthentication(authenticationToken);
        //放行
        filterChain.doFilter(httpServletRequest,httpServletResponse);
    }
}
