package space.anyi.BI.service.impl;

import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.util.StringUtils;
import space.anyi.BI.constant.SystemConstants;
import space.anyi.BI.entity.LoginUserDetails;
import space.anyi.BI.entity.ResponseResult;
import space.anyi.BI.entity.User;
import space.anyi.BI.entity.dto.UserDTO;
import space.anyi.BI.entity.vo.PageVO;
import space.anyi.BI.entity.vo.UserInfo;
import space.anyi.BI.entity.vo.UserVO;
import space.anyi.BI.exception.SystemException;
import space.anyi.BI.service.UserService;
import space.anyi.BI.mapper.UserMapper;
import org.springframework.stereotype.Service;
import space.anyi.BI.util.BeanCopyUtil;
import space.anyi.BI.util.JwtUtil;
import space.anyi.BI.util.RedisCache;

import javax.annotation.Resource;
import java.util.*;

/**
* @author 杨逸
* @description 针对表【user(用户表)】的数据库操作Service实现
* @createDate 2024-11-28 19:46:42
*/
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements UserService{
    @Resource
    private  AuthenticationManager authenticationManager;
    @Resource
    private  RedisCache redisCache;
    @Resource
    private  PasswordEncoder passwordEncoder;
    @Override
    public ResponseResult login(User user) {
        //认证
        UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(user.getUserAccount(), user.getUserPassword());
        Authentication authentication = authenticationManager.authenticate(authenticationToken);
        if (Objects.isNull(authentication)){
            throw new SystemException(ResponseResult.AppHttpCodeEnum.LOGIN_ERROR);
        }
        //生成token
        LoginUserDetails principal = (LoginUserDetails) authentication.getPrincipal();
        Long id = principal.getUser().getId();
        String jwt = JwtUtil.createJWT(id.toString());
        //todo:权限
        //存入redis
        redisCache.setCacheObject(SystemConstants.LOGIN_USER_REDIS_KEY+id,principal);
        //返回结果
        Map<String,Object> map = new HashMap<>();
        UserInfo userInfo = new UserInfo(principal.getUser().getUserAccount(),principal.getUser().getUserName(),principal.getUser().getUserAvatar(),principal.getUser().getUserRole());
        map.put("token",jwt);
        map.put("userInfo",userInfo);
        return ResponseResult.okResult(map);
    }

    @Override
    public void addUser(UserDTO userDTO) {
        User user = BeanCopyUtil.copyBean(userDTO, User.class);
        //密码加密
        user.setUserPassword(passwordEncoder.encode(user.getUserPassword()));
        //生成ID
        long id = IdUtil.getSnowflake(1, 1).nextId();
        user.setId(id);
        save(user);
    }

    @Override
    public PageVO pageByUsernameAndUseraccount(Integer currentPage, Integer pageSize, String userName, String userAccount, String userRole) {
        List<UserVO> userVOList = new ArrayList<>();
        LambdaQueryWrapper<User> lambdaQueryWrapper = new LambdaQueryWrapper<User>()
                .like(StringUtils.hasText(userName), User::getUserName, userName)
                .eq(StringUtils.hasText(userAccount), User::getUserAccount, userAccount)
                .eq(StringUtils.hasText(userRole), User::getUserRole, userRole);
        Page<User> page = page(new Page<User>(currentPage,pageSize), lambdaQueryWrapper);
        for (User user : page.getRecords()) {
            UserVO userVO = BeanCopyUtil.copyBean(user, UserVO.class);
            userVO.setId(user.getId().toString());
            userVOList.add(userVO);
        }
        return new PageVO(userVOList,page.getTotal());
    }

    @Override
    public ResponseResult register(User user) {
        //用户账号是否存在
        if (userAccountExist(user.getUserAccount())){
            throw new SystemException(ResponseResult.AppHttpCodeEnum.USERNAME_EXIST);
        }
        //密码加密
        user.setUserPassword(passwordEncoder.encode(user.getUserPassword()));
        //生成ID
        long id = IdUtil.getSnowflake(1, 1).nextId();
        user.setId(id);
        //存入数据库
        save(user);
        return ResponseResult.okResult();
    }
    /**
     * 判断用户账号是否已被注册
     * @param userAccount
     * @return
     */
    private boolean userAccountExist(String userAccount) {
        LambdaQueryWrapper<User> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(User::getUserAccount,userAccount);
        if (count(queryWrapper)>0) {
            return true;
        }
        return false;
    }
}




